@@ -12,9 +12,6 @@ import Control.Exception.Lifted
 import Test.Tasty as X
 import Test.Tasty.HUnit as X
 import Test.Tasty.TH as X
-import Test.HUnit as X hiding (Test)
-
-import Control.Concurrent.Async.Lifted as X
 
 value :: Int
 value = 42
